# snappy-proposed-selftest-agent
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import argparse
import configparser
import kombu
import logging
import os

from kombu.pools import producers

from uservice_utils.logging import (
    configure_service_logging,
    ExtraLogger,
)

from snappy_proposed_selftest_agent import constants
from snappy_proposed_selftest_agent.worker import CoreSelftestAgentWorker


class CoreImageResultPublisher(object):
    """A callable that can notify a results exchange that the test is running

    This will be a fanout exchange so multiple consumers can pick up these
    messages and decide whether to take action or ignore them.
    """

    def __init__(self, connection):
        self.connection = connection
        self.exchange = kombu.Exchange(
            constants.RESULTS_EXCHANGE, type='fanout')

    def __call__(self, payload):
        """Take 'payload' and enqueue it on the rabbit exchange."""
        with producers[self.connection].acquire(block=True) as producer:
            publisher = self.connection.ensure(
                producer,
                producer.publish,
                max_retries=3
            )
            publisher(
                payload, exchange=self.exchange, declare=[self.exchange])


class CoreImageBuilder(object):
    """A callable that knows how to queue message for the image builder

    For now we're using a simple queue, but this can easily be extended to use
    a full-blown topic exchange in the future.
    """

    def __init__(self, connection):
        self.connection = connection

    def __call__(self, payload):
        """Take 'payload' and enqueue it on the rabbit queue."""
        queue = self.connection.SimpleQueue(constants.IMAGE_BUILD_QUEUE)
        queue.put(payload)
        queue.close()


def parse_args():
    parser = argparse.ArgumentParser(description='Core Selftest Agent.')
    parser.add_argument(
        '-c',
        '--conf',
        default='snappy-proposed-service.conf',
        help='Configuration file path'
    )
    args = parser.parse_args()
    return args


def read_config(conf='snappy-proposed-service.conf'):
    # Load configuration options.
    config = configparser.ConfigParser()
    config.read(conf)
    return config


def get_logger(name, extra={}):
    if logging.getLoggerClass() != ExtraLogger:
        logging.setLoggerClass(ExtraLogger)
    logger = logging.getLogger(name)
    e = constants.LOGGING_EXTRA.copy()
    e.update(extra)
    logger.set_extra_args(e)
    return logger


def main():
    args = parse_args()
    config = read_config(args.conf)
    log_path = os.path.abspath(
        os.path.join(__file__,
                     '../../logs/snappy-proposed-selftest-agent.log'))
    configure_service_logging(
        log_path,
        config['logstash'] if 'logstash' in config else None
    )
    logger = logging.getLogger(__name__)
    logger.info('Started!', extra=constants.LOGGING_EXTRA)

    ampq_uris = config.get('amqp', 'uris').split()
    try:
        with kombu.Connection(ampq_uris) as connection:
            monitor = CoreSelftestAgentWorker(
                connection,
                config,
                CoreImageBuilder(connection),
                CoreImageResultPublisher(connection),
            )
            monitor.run()
    except KeyboardInterrupt:
        print("Bye!")
